@@ -16,7 +16,7 @@
 import locale
 
 from .__about__ import __version__
-from .iro import (Iro, Color, Style, ColorRGB, Color256, Font)
+from .iro import (Iro, Color, Style, ColorRGB, Color256, Font, IroElement)
 
 locale.setlocale(locale.LC_ALL, '')
 
@@ -28,4 +28,5 @@ __all__ = [
     "ColorRGB",
     "Color256",
     "Font",
+    "IroElement",
 ]
